@@ -63,11 +63,17 @@ suppressMessages(summary(opt))
 ## -----------------------------------------------------------------------------
 coef(opt$gating)
 
-## -----------------------------------------------------------------------------
-plot(opt, type="clusters")
+## ---- eval=FALSE--------------------------------------------------------------
+#  plot(opt, type="clusters")
 
-## -----------------------------------------------------------------------------
-plot(opt, type="mean")
+## ---- echo=FALSE--------------------------------------------------------------
+knitr::include_graphics("MVAD_Clusters.png")
+
+## ---- eval=FALSE--------------------------------------------------------------
+#  plot(opt, type="mean")
+
+## ---- echo=FALSE--------------------------------------------------------------
+knitr::include_graphics("MVAD_Mean.png")
 
 ## ---- fig.height=8.5----------------------------------------------------------
 plot(opt, type="dbsvals")
